-- *************************************************************
-- HowMany, a simple, addon to modify loot messages to tell you
-- how many you now have of the items you just looted.  Can
-- also tell you how much money you have as you loot.
-- By DemonCroc
-- *************************************************************

HowMany = LibStub("AceAddon-3.0"):NewAddon("HowMany", "AceConsole-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("HowMany")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local CreateFrame = _G.CreateFrame
local UIParent = _G.UIParent

--  ************************************************************
-- All of this first part is setting up the configuration
--  ************************************************************

local optFrame

local options = {
	args = { --  options are registered here into blizzard interface
		type = "group",
		icon = '',	
		name = L["HowMany"],	
		args = {
			Spacer = {
				type = "header",
				order = 1,
				name = L["HMLoot"]
			},
			-- We will place module options here, titled by module name
			-- Profile member will get added here by loading it from the db
		}
	}	
}


-- Which modules do we want enabled by default?  This is tied to the HowMany database, so it is saved across loads
local defaultModuleStatus = {
	profile = {
		modules = {
			["HMLoot"] = true,
			["Parser"] = true,
		}
	}
}


HowMany:SetDefaultModuleState(false)

local optionFrames = {}
local ACD3 = LibStub("AceConfigDialog-3.0")

-- Key is currency ID number
-- Value is currency name
HowMany.currencyInfo = {}

-- *******************
-- A bit of setup here, but still mostly configuration related stuff
-- *******************
function HowMany:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HowManyDB", defaultModuleStatus, "Default")

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HowMany", options.args)	
	optFrame = ACD3:AddToBlizOptions("HowMany", nil, nil)
	
	for rawModName, theMod in self:IterateModules() do

		-- check if the module has options
		local t
		if theMod.GetOptions then
			local modNameToUse = (theMod.modName or rawModName)
			t = theMod:GetOptions()
			options.args.args[modNameToUse] = t
		end
	end	
	
	-- Fetch currency information
	local id, name
	for id = 1, 500 do name = GetCurrencyInfo(id); if name and name ~= "" then HowMany.currencyInfo[id] = name; end end

	self:RegisterChatCommand("howmany", "OpenConfig")
	
	self.db.RegisterCallback(self, "OnProfileChanged", "SetUpdateConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "SetUpdateConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "SetUpdateConfig")
	
end

-- called by the /howmany command.  Opens to the howmany config page in the blizzard options
function HowMany:OpenConfig(input)
	--InterfaceOptionsFrame_OpenToCategory(HowMany.lastConfig)
	InterfaceOptionsFrame_OpenToCategory(optFrame)

end

-- This code deals with enabling/disabling modules as the user clicks on them.
do
	local timer, t = nil, 0
	local function update()
		t = t + arg1
		if t > 0.5 then
			timer:SetScript("OnUpdate", nil)
			HowMany:UpdateConfig()
		end
	end
	function HowMany:SetUpdateConfig() 
		t = 0
		timer = timer or CreateFrame("Frame", nil, UIParent)
		timer:SetScript("OnUpdate", update)
	end
end

function HowMany:UpdateConfig()
	for k, v in self:IterateModules() do
		if v:IsEnabled() then
			v:Disable()
			v:Enable()
		end
	end
end

--  ************************************************************
--  ok, finally time to get down to work
--  ************************************************************
function HowMany:OnEnable()

--	HowMany:Print(L["Welcome to HowMany! Type /howmany to configure."])

	for k, v in self:IterateModules() do
		if self.db.profile.modules[k] ~= false then
			v:Enable()
		end
	end
	

	
	if not options.args.args.Profiles then
 		options.args.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	end
end

function HowMany:OnDisable()
end

function HowMany.DebugPrint(prntstr)
--	HowMany:Print(prntstr)
end