local mod = HowMany:NewModule("HMLoot", "AceHook-3.0", "AceEvent-3.0", "AceTimer-3.0" )
local L = LibStub("AceLocale-3.0"):GetLocale("HowMany")
mod.modName = L["HMLoot"]
mod.canBeDisabled = true;

-- Find my parser module.  If it's not there, then, well, I can't do anything
local ParseMod = HowMany:GetModule("Parser")


local UnitName = _G.UnitName
local pairs = _G.pairs
local select = _G.select
local type = _G.type
local gsub = _G.string.gsub
local string_find = string.find
local ChatFrame_GetMessageEventFilters = _G.ChatFrame_GetMessageEventFilters
local HMDebPrint = HowMany.DebugPrint

-- contains items we've received recently, and are holding to see if any more come in.
-- Keyed by item link.  Value is a table, with keys "TimeReceived" and "ParserEvent", "TimerHandle"
local ItemHash = {}

-- Dynamically created frames for receiving events and tooltip info.
local eventFrame


-- Money strings.  Take copies of the global strings, remove the parameters.  This gives us, essentially, translated words for Gold, silver, copper
local GOLD = gsub(GOLD_AMOUNT, "%%d ", "")
local SILVER = gsub(SILVER_AMOUNT, "%%d ", "")
local COPPER = gsub(COPPER_AMOUNT, "%%d ", "")


local player = UnitName("player")
local defaults = {
	profile = {
		trackMoney = true,
		moneyAsWords = false,
		moneyAsColors = true,
		trackItems = true
	}
}

local options = {
	type = "group",
	name = L["HMLoot Options"],
	order = 1,
	childGroups = 'tree',
	args = {
		Enable = {
			type = "toggle", 
			name = L["Enable "] .. "module HMLoot",
			desc = L["Enable "] .. "module HMLoot",
			order = 11,
			get = function()
				return HowMany.db.profile.modules["HMLoot"] ~= false or false
			end,
			set = function(_, v)
				HowMany.db.profile.modules["HMLoot"] = v
				if v then
					HowMany:EnableModule("HMLoot")											
				else
					HowMany:DisableModule("HMLoot")					
				end
			end
		},
		MoneyOptions = {
			type = "group",
			name = L["Money Loot Options"],
			order = 1,
			args = {
				money = {
					type = "toggle",
					name = L["Show Total Money"],
					desc = L["Show total money on loot messages"],
					descStyle = "inline",
					order = 5,
					get = function()
						return mod.db.profile.trackMoney
					end,
					set = function(info, v)
						mod.db.profile.trackMoney = v
					end
				},
				moneywords = {
					type = "toggle",
					name = L["Show with words"],
					desc = L["Show denomination with words.  321 Gold 45 Silver 67 Copper"],
					descStyle = "inline",
					order = 7,
					get = function()
						return mod.db.profile.moneyAsWords
					end,
					set = function(info, v)
						mod.db.profile.moneyAsWords = v
					end
				},
				moneycolor = {
					type = "toggle",
					name = L["Colorize money loots"],
					desc = L["Show total money using colors. |cffffd700321|r |cff80808045|r |cffeda55f67|r"],
					descStyle = "inline",
					order = 8,
					get = function()
						return mod.db.profile.moneyAsColors
					end,
					set = function(info, v)
						mod.db.profile.moneyAsColors = v
					end
				}
			}
		},
		ItemOptions = {
			type = "group",
			name = L["Item Loot Options"],
			order = 2,
			args = {			
				items = {
					type = "toggle",
					name = L["Show Bags Total"],
					desc = L["Show total in bags on loot messages"],
					descStyle = "inline",
					order = 6,
					get = function()
						return mod.db.profile.trackItems
					end,
					set = function(info, v)
						mod.db.profile.trackItems = v
					end
				}
			}
		},
		CurrencyOptions = {
			type = "group",
			name = L["Currency Loot Options"],
			order = 2,
			args = {			
				items = {
					type = "toggle",
					name = L["Show New Total"],
					desc = L["Show new currency total on loot messages"],
					descStyle = "inline",
					order = 6,
					get = function()
						return mod.db.profile.trackCurrency
					end,
					set = function(info, v)
						mod.db.profile.trackCurrency = v
					end
				}
			}
		},
		CombineOptions = {
			type = "group",
			name = L["Combine Messages"],
			order = 2,
			args = {			
				items = {
					type = "toggle",
					name = L["Combine Similar Loot Messages"],
					desc = L["Combine multiple loots for the same item in a short timeframe together into a single message."],
					descStyle = "inline",
					order = 6,
					get = function()
						return mod.db.profile.trackCurrency
					end,
					set = function(info, v)
						mod.db.profile.trackCurrency = v
					end
				}
			}
		}
		

	}
}

-- These are the events that I'm interested in parsing.
local valid_events = {
	CHAT_MSG_LOOT = true,
	CHAT_MSG_MONEY = true,
	CHAT_MSG_CURRENCY = true
}

-- *********************************************************
-- *********************************************************
function mod:OnInitialize()
	self.db = HowMany.db:RegisterNamespace("HMLoot", defaults)
	
	eventFrame = CreateFrame("Frame")
	eventFrame:Hide()
	eventFrame:SetScript("OnEvent", OnEvent)

end


-- *********************************************************
-- *********************************************************
function mod:CurrencyTimerCallback(tableKey)

if ( ItemHash[tableKey] ) then  -- ok, we found it
    
    -- Need to inject an event now..
    if ( mod.hooks[ItemHash[tableKey]["ParserEvent"]["OriginalFrame"]] ) then
		mod.hooks[ItemHash[tableKey]["ParserEvent"]["OriginalFrame"]]:AddMessage( ItemHash[tableKey]["ParserEvent"]["OriginalFrame"], ItemHash[tableKey]["ParserEvent"]["OriginalText"] )
	end
	
	ItemHash[tableKey] = nil
end

end

-- *********************************************************
-- *********************************************************
local function HandleCurrency(parserEvent)
--HMDebPrint("In HandleCurrency.." .. parserEvent.moneyString .. " "  .. parserEvent.amount )



if ( not mod.db.profile.trackCurrency ) then return end

local idx, cname, ccount, citemID
for idx = 1, GetCurrencyListSize() do

	cname, _, _, _, _, ccount, _, _, citemID = GetCurrencyListInfo(idx)
	
	local matchEnd = string_find(parserEvent.moneyString, cname)

	-- Check if a match was found. 
	if (matchEnd) then
--		if ( ItemHash[parserEvent.moneyString] ) then -- we've recently seen one of these.. combine

			ItemHash[parserEvent.moneyString] = nil  -- We don't have to worry about the timer any more, so just forget it.
			parserEvent.MessageCombined = false  -- we're not combining this message.  Just let it go through
			parserEvent.resultString = " You now have " .. ccount .. "."

--		else
--
--			ItemHash[parserEvent.moneyString] = {}
--			ItemHash[parserEvent.moneyString]["TimeReceived"] = GetTime()
--			ItemHash[parserEvent.moneyString]["ParserEvent"] = parserEvent
--
---- Start the timer here..
--			ItemHash[parserEvent.moneyString]["TimerHandle"] = mod.ScheduleTimer("CurrencyTimerCallback", .5, parserEvent.moneyString)
--						
--			parserEvent.MessageCombined = true  -- we're combining this message, so kill the event here.
--		end
		break
	end
end

end


-- *********************************************************
-- *********************************************************
local function HandleMoney(parserEvent)
--HMDebPrint("In HandleMoney.." .. parserEvent.moneyString )

if ( not mod.db.profile.trackMoney ) then return end

local mstart, mend, capCopper, capSilver, capGold

-- pull the amount of gold, silver, copper in this loot message
mstart, mend, capCopper = string_find(parserEvent.moneyString,"(%d+) " .. COPPER)
capCopper = capCopper or 0
mstart, mend, capSilver = string_find(parserEvent.moneyString,"(%d+) " .. SILVER)
capSilver = capSilver or 0
mstart, mend, capGold = string_find(parserEvent.moneyString,"(%d+) " .. GOLD)
capGold = capGold or 0

-- Add what was just looted to the player total
 local playerMoney = GetMoney() + capCopper + 100*capSilver + 100*100*capGold
 local playerGold = math.floor(playerMoney / (100 * 100))
 playerMoney = playerMoney - 100*100 * playerGold
 local playerSilver = math.floor(playerMoney / 100)
 local playerCopper = playerMoney % 100

-- Format the result
  if ( mod.db.profile.moneyAsWords ) then
      playerGold = tostring(playerGold) .. " " .. GOLD
      playerSilver = tostring(playerSilver) .. " " .. SILVER
      playerCopper = tostring(playerCopper) .. " " .. COPPER
  end
 
  if ( mod.db.profile.moneyAsColors ) then
  	parserEvent.resultString = ". You now have |cffffd700" .. playerGold .. "|r |cff808080" .. playerSilver .. "|r " .. "|cffeda55f" .. playerCopper .. "|r"
  else
    parserEvent.resultString = ". You now have " .. playerGold .. " " .. playerSilver .. " " .. playerCopper
  end
 
 parserEvent.MessageCombined = false  -- we're not combining this message.  Just let it go through
end


-- *********************************************************
-- *********************************************************
local function HandleItems(parserEvent)
--HMDebPrint("In HandleItems" )
 -- Created items are subject to a race condition.  Sometimes they're already in inventory when we get notified,
 -- sometimes they're not.  So just ignore them.
 if (parserEvent.isCreate) then return end

 -- Get information about the looted item.
 local itemLink = parserEvent.itemLink
 
 -- Get the number of items already existing in inventory and add the amount
 -- looted to it if the item wasn't the result of a conjure.
 local numLooted = parserEvent.amount or 1
 local numItems = GetItemCount(itemLink) or 0  
 local numTotal = numItems + numLooted


--if ( ItemHash[parserEvent.itemLink] ) then -- we've recently seen one of these.. combine

	ItemHash[parserEvent.itemLink] = nil  -- We don't have to worry about the timer any more, so just forget it.
	parserEvent.MessageCombined = false  -- we're not combining this message.  Just let it go through

	parserEvent.resultString = " You now have " .. numTotal .. "."
--else
--
--	ItemHash[parserEvent.itemLink] = {}
--	ItemHash[parserEvent.itemLink]["TimeReceived"] = GetTime()
--	ItemHash[parserEvent.itemLink]["ParserEvent"] = parserEvent
--
---- Start the timer here..
--	ItemHash[parserEvent.itemLink]["TimerHandle"] = mod.ScheduleTimer(mod, "CurrencyTimerCallback", .5, parserEvent.itemLink)
--	
--	
--	parserEvent.MessageCombined = true  -- we're combining this message, so kill the event here.
--end

  
end


-- *********************************************************
-- *********************************************************
local function ParserEventsHandler(parserEvent,msgText)
  
  --HMDebPrint("In ParserEventsHandler" )
  
 -- Ignore the event if it isn't for the player or not a loot event.
 if (parserEvent.recipientUnit ~= "player" or (parserEvent.eventType ~= "loot" and parserEvent.eventType ~= "currency")) then return end

 -- Call the correct handler for the loot type.
 if (parserEvent.eventType == "currency") then HandleCurrency(parserEvent) elseif (parserEvent.isMoney) then HandleMoney(parserEvent) elseif (parserEvent.itemLink) then HandleItems(parserEvent) end
end


-- *********************************************************
-- *********************************************************
function mod:OnEnable()
	
	HMDebPrint("In HMLoot:OnEnable" )
	
 	for event in pairs(valid_events) do
 	 eventFrame:RegisterEvent(event)
 	end
	
	
	-- Set the raw hooks on the chat frames.  Raw hooks allow us to
	-- intercept the message and possibly modify it before it reaches
	-- its intended destination (the chat frame)
	for i = 1, NUM_CHAT_WINDOWS do
		local cf = _G["ChatFrame" .. i]
		if cf ~= COMBATLOG then
			self:RawHook(cf, "AddMessage", true)
			--HMDebPrint("Hooked " .. i )
		end
	end
	
	 eventFrame:Show()

end


-- *********************************************************
-- *********************************************************
function mod:OnDisable()
 -- Stop receiving updates.
 eventFrame:Hide()
 eventFrame:UnregisterAllEvents()

 mod.CancelAllTimers()
end




-- *********************************************************
-- *********************************************************
function mod:AddMessage(frame, text, ...)

--HMDebPrint("In HMLoot:AddMessage" )
	local pEvent = nil
	
	if text and type(text) == "string" then 
		
		if ( ParseMod ~= nil ) then
			
			pEvent = ParseMod.ParseRawChatEvent(text)
			
--			text = text .. "1"  -- DEBUG
			
			if ( pEvent ) then
--				text = text .. "2"  -- DEBUG
				ParserEventsHandler(pEvent,text)
				
--				text = text .. "3"  -- DEBUG
				if ( pEvent.MessageCombined ) then
					pEvent.OriginalFrame = frame  -- Save this in case we're combining messages
--					text = text .. "4"  -- DEBUG
				end
				
				if ( pEvent.resultString ) then
					text = text .. pEvent.resultString
--					text = text .. "5"  -- DEBUG
				end
			end
		end
	end 	
	
	
	
		
	-- If we're combining this message, then do not pass it on..
	if ( ParseMod ~= nil and pEvent and pEvent.MessageCombined ) then
--		text = text .. pEvent["OriginalText"] -- "6"  -- DEBUG
		return self.hooks[frame].AddMessage(frame, text, ...) -- DEBUG
--		return
	else
		return self.hooks[frame].AddMessage(frame, text, ...)
	end
end

-- *********************************************************
-- Present some information to Ace
-- *********************************************************
function mod:Info()
	return L["When looting money or items, show how much/many you now have."]
end

function mod:GetOptions()
	return options
end

